// found in Arduino MIDI Library examples
// modified for Mega32 by jean-claude.feltes@education.lu
// Use at least 2 of these and mix the signal to get an interesting sound.

/*
Potentiometer on A0: detune factor 1...2  (unison ... octave)
// Buttons: 1: Unison
            2: Octave
            3: Fith
            4: Fourth   interval
*/

#include <MIDI.h>
#include "noteList.h"
#include "pitches.h"

MIDI_CREATE_DEFAULT_INSTANCE();


// This example shows how to make a simple synth out of an Arduino, using the
// tone() function. It also outputs a gate signal for controlling external
// analog synth components (like envelopes).

//static const unsigned sGatePin     = 13;      // = B.5     (LED)  // original
static const unsigned sGatePin     = 5;      // = D.5     (LED)       = B.5 for Mega32
static const unsigned sAudioOutPin = 10;      // = B.2  for Mega8     = D.2 for Mega32              
const int button1 = 0;     //B.0       for Mega32
const int button2 = 1;     //B.1
const int button3 = 2;     //B.2
const int button4 = 3;     //B.3 
static const unsigned sMaxNumNotes = 16;
MidiNoteList<sMaxNumNotes> midiNotes;

//CHANGED
float detuneFactor = 1;
float detuneFactor1 = 1;
  
static const unsigned analogInPin = A0;
//END CHANGED
// -----------------------------------------------------------------------------
void setup()
{
    pinMode(sGatePin,     OUTPUT);
    pinMode(sAudioOutPin, OUTPUT);
      pinMode(button1, INPUT_PULLUP);
       pinMode(button2, INPUT_PULLUP);
        pinMode(button3, INPUT_PULLUP);
         pinMode(button4, INPUT_PULLUP);
    MIDI.setHandleNoteOn(handleNoteOn);
    MIDI.setHandleNoteOff(handleNoteOff);
    MIDI.begin();

        
}
//------------------------------------------------------------------------------------

inline void handleGateChanged(bool inGateActive)
{
    digitalWrite(sGatePin, inGateActive ? HIGH : LOW);
}

inline void pulseGate()
{
    handleGateChanged(false);
    delay(1);
    handleGateChanged(true);
}

// -----------------------------------------------------------------------------

// ADDED TO DECOUPLE SOUND CREATION FROM REST

void createTone(float frequency){
    tone(sAudioOutPin, frequency);
}

void switchOffTone(){
    noTone(sAudioOutPin);
}

//----------------------------------------------------------------------------

void handleNotesChanged(bool isFirstNote = false)
{
    if (midiNotes.empty())
    {
        handleGateChanged(false);
        switchOffTone();          // Remove to keep oscillator running during envelope release.
    }
    else
    {
        // Possible playing modes:
        // Mono Low:  use midiNotes.getLow
        // Mono High: use midiNotes.getHigh
        // Mono Last: use midiNotes.getLast

        byte currentNote = 0;
        if (midiNotes.getLast(currentNote))
        {   
            //CHANGED
            float f = sNotePitches[currentNote];
            f = detuneFactor * f;
            createTone(f);
            //END CHANGED

            
            if (isFirstNote)
            {
                handleGateChanged(true);
            }
            else
            {
                pulseGate(); // Retrigger envelopes. Remove for legato effect.
            }
        }
    }
}

// -----------------------------------------------------------------------------

void handleNoteOn(byte inChannel, byte inNote, byte inVelocity)
{
    const bool firstNote = midiNotes.empty();
    midiNotes.add(MidiNote(inNote, inVelocity));
    handleNotesChanged(firstNote);
}

void handleNoteOff(byte inChannel, byte inNote, byte inVelocity)
{
    midiNotes.remove(inNote);
    handleNotesChanged();
}

// -----------------------------------------------------------------------------
void readDetuneFactor(){
  // set detuneFactor to 1 / 2 /1.5 with buttons
  // and variate it with potentiometer by factor 1 ... 2
  
  int potValue = analogRead(analogInPin);
  float detuneFactor2 = potValue/1023.0 + 1;
      
  if (digitalRead(button1) == 0){  
      detuneFactor1 = 1.0 ;               //unison
      }
  if (digitalRead(button2) == 0){         // octave
      detuneFactor1 = 2.0 ;   
   }
   if (digitalRead(button3) == 0){        // fifth
      detuneFactor1 = 1.5;
      }
   if (digitalRead(button4) == 0){        // fourth
      detuneFactor1 = 1.25;
      }   

   detuneFactor = detuneFactor1 * detuneFactor2;   
   
}
//------------------------------------------------------------------------------



void loop(){
     MIDI.read();
     readDetuneFactor();
     
}
